@@ -25,6 +25,7 @@ import dxpy
 from dxpy.scripts import dx_build_app
 from dxpy_testutil import DXTestCase
 import dxpy_testutil as testutil
+from dxpy.packages import requests
 
 @contextmanager
 def chdir(dirname=None):
@@ -646,6 +647,49 @@ class TestDXClient(DXTestCase):
         with self.assertSubprocessFailure(stderr_regexp='JSON', exit_code=3):
             run("dx run " + applet_id + " --extra-args not-a-JSON-string")
 
+
+@unittest.skipUnless(testutil.TEST_HTTP_PROXY,
+                     'skipping HTTP Proxy support test that needs squid3')
+class TestHTTPProxySupport(DXTestCase):
+    def setUp(self):
+        squid_wd = os.path.join(os.path.dirname(__file__), 'http_proxy')
+        self.proxy_process = subprocess.Popen(['squid3', '-N', '-f', 'squid.conf'], cwd=squid_wd)
+        time.sleep(1)
+
+        print "Waiting for squid to come up..."
+        t = 0
+        while True:
+            try:
+                if requests.get("http://localhost:3129").status_code == requests.codes.bad_request:
+                    if self.proxy_process.poll() is not None:
+                        # Got a response on port 3129, but our proxy quit with an error, so it must be another process.
+                        raise Exception("Tried launching squid, but port 3129 is already bound")
+                    print "squid is up"
+                    break
+            except requests.exceptions.RequestException:
+                pass
+            time.sleep(0.5)
+            t += 1
+            if t > 16:
+                raise Exception("Failed to launch Squid")
+
+        self.proxy_env_no_auth = os.environ.copy()
+        self.proxy_env_no_auth["HTTP_PROXY"] = "http://localhost:3129"
+        self.proxy_env_no_auth["HTTPS_PROXY"] = "http://localhost:3129"
+
+        self.proxy_env = os.environ.copy()
+        self.proxy_env["HTTP_PROXY"] = "http://proxyuser:proxypassword@localhost:3129"
+        self.proxy_env["HTTPS_PROXY"] = "http://proxyuser:proxypassword@localhost:3129"
+
+    def test_proxy(self):
+        run("dx find projects", env=self.proxy_env)
+        with self.assertSubprocessFailure(stderr_regexp="407 Proxy Authentication Required"):
+            run("dx find projects", env=self.proxy_env_no_auth)
+
+    def tearDown(self):
+        self.proxy_process.terminate()
+
+
 class TestDXBuildApp(DXTestCase):
     def setUp(self):
         self.temp_file_path = tempfile.mkdtemp()
@@ -777,6 +821,25 @@ class TestDXBuildApp(DXTestCase):
         with self.assertSubprocessFailure(stderr_regexp="Expected runSpec\.execDepends to"):
             run("dx build --json " + app_dir)
 
+    def test_invalid_authorized_users(self):
+        app_spec = {
+            "name": "invalid_authorized_users",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "1.0.0",
+            "authorizedUsers": "PUBLIC"
+            }
+        app_dir = self.write_app_directory("invalid_authorized_users", json.dumps(app_spec), "code.py")
+        with self.assertSubprocessFailure(stderr_regexp='Expected authorizedUsers to be a list of strings'):
+            run("dx build --json " + app_dir)
+
+        app_spec["authorizedUsers"] = ["foo"]
+        app_dir = self.write_app_directory("invalid_authorized_users_2", json.dumps(app_spec), "code.py")
+        with self.assertSubprocessFailure(stderr_regexp='contains an entry which is not'):
+            run("dx build --json " + app_dir)
+
     def test_duplicate_keys_in_spec(self):
         app_spec = {
             "name": "test_duplicate_keys_in_spec",
@@ -862,6 +925,180 @@ class TestDXBuildApp(DXTestCase):
         run("dx build --create-app --json " + app_dir)
         self.assertEquals(json.loads(run("dx api " + app_id + " listCategories"))["categories"], ['B'])
 
+    @unittest.skipUnless(testutil.TEST_CREATE_APPS,
+                         'skipping test that would create apps')
+    def test_update_app_authorized_users(self):
+        app0_spec = {
+            "name": "update_app_authorized_users",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "0.0.1"
+            }
+        app1_spec = {
+            "name": "update_app_authorized_users",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "1.0.0",
+            "authorizedUsers": []
+            }
+        app2_spec = {
+            "name": "update_app_authorized_users",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "1.0.1",
+            "authorizedUsers": ["PUBLIC"]
+            }
+        app_dir = self.write_app_directory("update_app_authorized_users", json.dumps(app0_spec), "code.py")
+        app_id = json.loads(run("dx build --create-app --json " + app_dir))['id']
+        self.assertEquals(json.loads(run("dx api " + app_id + " listAuthorizedUsers"))["authorizedUsers"], ["PUBLIC"])
+        shutil.rmtree(app_dir)
+        self.write_app_directory("update_app_authorized_users", json.dumps(app1_spec), "code.py")
+        run("dx build --create-app --json " + app_dir)
+        self.assertEquals(json.loads(run("dx api " + app_id + " listAuthorizedUsers"))["authorizedUsers"], [])
+        shutil.rmtree(app_dir)
+        self.write_app_directory("update_app_authorized_users", json.dumps(app2_spec), "code.py")
+        run("dx build --create-app --json " + app_dir)
+        self.assertEquals(json.loads(run("dx api " + app_id + " listAuthorizedUsers"))["authorizedUsers"], ["PUBLIC"])
+
+    @unittest.skipUnless(testutil.TEST_CREATE_APPS,
+                         'skipping test that would create apps')
+    def test_dx_add_list_remove_users(self):
+        '''
+        This test is for some other dx subcommands, but it's in this
+        test suite to take advantage of app-building methods.
+        '''
+        # Only create the app if it's not available already (makes
+        # local testing easier)
+        try:
+            app_desc = dxpy.api.app_describe("app-test_dx_users", {})
+            app_id = app_desc["id"]
+            # reset users to default list
+            run("dx remove users app-test_dx_users " + " ".join(app_desc["authorizedUsers"]))
+            run("dx add users app-test_dx_users PUBLIC")
+        except:
+            app_id = None
+        if app_id is None:
+            app_spec = {
+                "name": "test_dx_users",
+                "dxapi": "1.0.0",
+                "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+                "inputSpec": [],
+                "outputSpec": [],
+                "version": "0.0.1"
+                }
+            app_dir = self.write_app_directory("test_dx_users", json.dumps(app_spec), "code.py")
+            app_id = json.loads(run("dx build --create-app --json " + app_dir))['id']
+        users = run("dx list users app-test_dx_users").strip()
+        self.assertEqual(users, "PUBLIC")
+        # use hash ID
+        run("dx remove users " + app_id + " PUBLIC")
+        users = run("dx list users app-test_dx_users").strip()
+        self.assertEqual(users, "")
+        # don't use "app-" prefix, duplicate and multiple members are fine
+        run("dx add users test_dx_users PUBLIC eve user-eve org-piratelabs")
+        users = run("dx list users app-test_dx_users").strip().split("\n")
+        self.assertEqual(len(users), 3)
+        self.assertIn("PUBLIC", users)
+        self.assertIn("user-eve", users)
+        self.assertIn("org-piratelabs", users)
+        run("dx remove users test_dx_users eve org-piratelabs")
+        # use version string
+        users = run("dx list users app-test_dx_users/0.0.1").strip()
+        self.assertEqual(users, 'PUBLIC')
+
+        # bad paths and exit codes
+        with self.assertSubprocessFailure(stderr_regexp='could not be resolved', exit_code=3):
+            run('dx add users nonexistentapp PUBLIC')
+        with self.assertSubprocessFailure(stderr_regexp='could not be resolved', exit_code=3):
+            run('dx list users app-nonexistentapp')
+        with self.assertSubprocessFailure(stderr_regexp='could not be resolved', exit_code=3):
+            run('dx remove users app-nonexistentapp/1.0.0 PUBLIC')
+        with self.assertSubprocessFailure(stderr_regexp='ResourceNotFound', exit_code=3):
+            run('dx add users test_dx_users org-nonexistentorg')
+        with self.assertSubprocessFailure(stderr_regexp='ResourceNotFound', exit_code=3):
+            run('dx add users test_dx_users nonexistentuser')
+        with self.assertSubprocessFailure(stderr_regexp='ResourceNotFound', exit_code=3):
+            run('dx add users test_dx_users piratelabs')
+
+        # ResourceNotFound is not thrown when removing things
+        run('dx remove users test_dx_users org-nonexistentorg')
+        run('dx remove users test_dx_users nonexistentuser')
+        run('dx remove users test_dx_users piratelabs')
+
+    @unittest.skipUnless(testutil.TEST_CREATE_APPS,
+                         'skipping test that would create apps')
+    def test_dx_add_list_remove_developers(self):
+        '''
+        This test is for some other dx subcommands, but it's in this
+        test suite to take advantage of app-building methods.
+        '''
+        # Only create the app if it's not available already (makes
+        # local testing easier)
+        try:
+            app_desc = dxpy.api.app_describe("app-test_dx_developers", {})
+            app_id = app_desc["id"]
+            my_userid = app_desc["createdBy"]
+            developers = dxpy.api.app_list_developers("app-test_dx_developers", {})["developers"]
+            # reset developers to default list
+            if len(developers) != 1:
+                run("dx remove developers app-test_dx_developers " +
+                    " ".join([dev for dev in developers if dev != my_userid]))
+        except:
+            app_id = None
+        if app_id is None:
+            app_spec = {
+                "name": "test_dx_developers",
+                "dxapi": "1.0.0",
+                "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+                "inputSpec": [],
+                "outputSpec": [],
+                "version": "0.0.1"
+                }
+            app_dir = self.write_app_directory("test_dx_developers", json.dumps(app_spec), "code.py")
+            app_desc = json.loads(run("dx build --create-app --json " + app_dir))
+            app_id = app_desc['id']
+            my_userid = app_desc["createdBy"]
+        developers = run("dx list developers app-test_dx_developers").strip()
+        self.assertEqual(developers, my_userid)
+        # use hash ID
+        run("dx add developers " + app_id + " eve")
+        developers = run("dx list developers app-test_dx_developers").strip().split("\n")
+        self.assertEqual(len(developers), 2)
+        self.assertIn(my_userid, developers)
+        # don't use "app-" prefix, duplicate, multiple, and non- members are fine
+        run("dx remove developers test_dx_developers PUBLIC eve user-eve org-piratelabs")
+        developers = run("dx list developers app-test_dx_developers").strip()
+        self.assertEqual(developers, my_userid)
+        # use version string
+        run("dx list developers app-test_dx_developers/0.0.1")
+
+        # bad paths and exit codes
+        with self.assertSubprocessFailure(stderr_regexp='could not be resolved', exit_code=3):
+            run('dx add developers nonexistentapp eve')
+        with self.assertSubprocessFailure(stderr_regexp='could not be resolved', exit_code=3):
+            run('dx list developers app-nonexistentapp')
+        with self.assertSubprocessFailure(stderr_regexp='could not be resolved', exit_code=3):
+            run('dx remove developers app-nonexistentapp/1.0.0 eve')
+        with self.assertSubprocessFailure(stderr_regexp='ResourceNotFound', exit_code=3):
+            run('dx add developers test_dx_developers nonexistentuser')
+        with self.assertSubprocessFailure(stderr_regexp='ResourceNotFound', exit_code=3):
+            run('dx add developers test_dx_developers piratelabs')
+
+        # ResourceNotFound is not thrown when removing things
+        run('dx remove developers test_dx_developers org-nonexistentorg')
+        run('dx remove developers test_dx_developers nonexistentuser')
+        run('dx remove developers test_dx_developers piratelabs')
+
+        # Raise an error if you try to add an org developer (currently unsupported by the API)
+        with self.assertSubprocessFailure(stderr_regexp='unsupported', exit_code=3):
+            run('dx add developers test_dx_developers org-piratelabs')
+
     @unittest.skipUnless(testutil.TEST_CREATE_APPS,
                          'skipping test that would create apps')
     def test_build_app_autonumbering(self):
@@ -914,6 +1151,39 @@ class TestDXBuildApp(DXTestCase):
             run("dx build " + app_dir)
         run("dx build --no-check-syntax " + app_dir)
 
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
+                         'skipping test that would run jobs')
+    def test_build_and_run_applet_remote(self):
+        app_spec = {
+            "name": "build_applet_remote",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+            "inputSpec": [
+                {"name": "in1", "class": "int"},
+            ],
+            "outputSpec": [
+                {"name": "out1", "class": "int"}
+            ],
+            "version": "1.0.0"
+            }
+        app_code = """import dxpy
+@dxpy.entry_point("main")
+def main(in1):
+    return {"out1": in1 + 1}
+"""
+        app_dir = self.write_app_directory(
+            'build_applet_remote', json.dumps(app_spec), code_filename='code.py', code_content=app_code)
+        remote_build_output = run('dx build --remote ' + app_dir).strip().split('\n')[-1]
+        # TODO: it would be nice to have the output of dx build --remote
+        # more machine readable (perhaps when --json is specified)
+        build_job_id = re.search('job-[A-Za-z0-9]{24}', remote_build_output).group(0)
+        build_job_describe = json.loads(run('dx describe --json ' + build_job_id))
+        applet_id = build_job_describe['output']['output_applet']['$dnanexus_link']
+        invocation_job_id = run('dx run --brief --yes ' + applet_id + ' -iin1=8675309').strip()
+        run('dx wait ' + invocation_job_id)
+        invocation_job_describe = json.loads(run('dx describe --json ' + invocation_job_id))
+        self.assertEquals(invocation_job_describe['output']['out1'], 8675310)
+
     def test_applet_help(self):
         app_spec = {
             "name": "applet_help",
@@ -937,6 +1207,28 @@ class TestDXBuildApp(DXTestCase):
         self.assertTrue("Optional: [-ioptional=(file)]" in applet_help)
         self.assertTrue("Mappings: mappings (gtable, type LetterMappings)" in applet_help)
 
+    def test_upload_resources(self):
+        run("dx mkdir /subfolder")
+        run("dx cd /subfolder")
+        app_spec = {
+            "name": "upload_resources",
+            "dxapi": "1.0.0",
+            "runSpec": {"file": "code.py", "interpreter": "python2.7"},
+            "inputSpec": [],
+            "outputSpec": [],
+            "version": "1.0.0"
+            }
+        app_dir = self.write_app_directory("upload_resources", json.dumps(app_spec), "code.py")
+        os.mkdir(os.path.join(app_dir, 'resources'))
+        with open(os.path.join(app_dir, 'resources', 'test.txt'), 'w') as resources_file:
+            resources_file.write('test\n')
+        new_applet = json.loads(run("dx build --json " + app_dir))
+        applet_describe = json.loads(run("dx describe --json " + new_applet["id"]))
+        resources_file = applet_describe['runSpec']['bundledDepends'][0]['id']['$dnanexus_link']
+        resources_file_describe = json.loads(run("dx describe --json " + resources_file))
+        # Verify that the bundled depends appear in the same folder.
+        self.assertEqual(resources_file_describe['folder'], '/subfolder')
+
 
 class TestDXBuildReportHtml(unittest.TestCase):
     js = "console.log('javascript');"
